#!/usr/bin/env python
# -*- coding: utf-8 -*-

import attr

from cryptodatahub.common.stores import (
    RootCertificate,
    RootCertificateParams,
    RootCertificateTrustConstraintAction,
)
from cryptodatahub.common.types import Base64Data, CryptoDataParamsBase

from cryptodatahub.common.fetcher import FetcherRootCertificateStore

from updaters.common import UpdaterBase


@attr.s
class RootCertificateStore(CryptoDataParamsBase):
    certificates = attr.ib(
        default={},
        validator=attr.validators.deep_mapping(
            key_validator=attr.validators.instance_of(Base64Data),
            value_validator=attr.validators.instance_of(RootCertificateTrustConstraintAction)
        )
    )


class UpdaterRootCertificateTrustStore(UpdaterBase):
    def __init__(self):
        super().__init__(  # pragma: no cover
            fetcher_class=FetcherRootCertificateStore,
            enum_class=RootCertificate,
            enum_param_class=RootCertificateParams,
        )


def main():
    UpdaterRootCertificateTrustStore()()  # pragma: no cover


if __name__ == '__main__':
    main()  # pragma: no cover



# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.tls.version import TlsVersion


class TestTlsVersion(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsVersion



# -*- coding: utf-8 -*-

import unittest

from test.common.classes import EnumIntTest


from cryptodatahub.common.exception import InvalidValue


class TestException(unittest.TestCase):
    def test_str(self):
        with self.assertRaisesRegex(InvalidValue, '0xa is not a valid str member name value') as context_manager:
            raise InvalidValue(10, str, 'member name')
        self.assertEqual(context_manager.exception.value, 10)

        with self.assertRaisesRegex(InvalidValue, '0xa is not a valid str') as context_manager:
            raise InvalidValue(10, str)
        self.assertEqual(context_manager.exception.value, 10)

        with self.assertRaisesRegex(InvalidValue, '0xa is not a valid EnumIntTest') as context_manager:
            raise InvalidValue(EnumIntTest.TEN, EnumIntTest)
        self.assertEqual(context_manager.exception.value, 10)

        with self.assertRaisesRegex(InvalidValue, '1.0 is not a valid float') as context_manager:
            raise InvalidValue(1.0, float)
        self.assertEqual(context_manager.exception.value, 1.0)



# -*- coding: utf-8 -*-

try:
    from unittest import mock
except ImportError:
    import mock

import csv
import datetime
import io
import os
import tarfile

from test.common.classes import TestClasses

from cryptodatahub.common.algorithm import Hash
from cryptodatahub.common.fetcher import (
    FetcherRootCertificateStore,
    FetcherRootCertificateStoreApple,
    FetcherRootCertificateStoreGoogle,
    FetcherRootCertificateStoreMicrosoft,
    FetcherRootCertificateStoreMozilla,
)
from cryptodatahub.common.stores import (
    CertificateTrustConstraint,
    RootCertificateBase,
    RootCertificateParams,
    RootCertificateTrustConstraintAction,
)
from cryptodatahub.common.types import Base64Data
from cryptodatahub.common.utils import HttpFetcher

from updaters.common import UpdaterBase
from updaters.trust_stores import RootCertificateStore


class TestRootCertificateBase(TestClasses.TestKeyBase):
    def setUp(self):
        super().setUp()

        self.public_key_x509_lets_encrypt = self._get_public_key_x509('letsencrypt_isrg_root_x1')
        self.public_key_x509_snakeoil_ca = self._get_public_key_x509('snakeoil_ca_cert')

    @staticmethod
    def _get_mock_data_mozilla(public_keys=(), options=None):
        mock_data = io.StringIO()
        dict_writer = csv.DictWriter(
            mock_data, FetcherRootCertificateStoreMozilla.CSV_FIELDS,
            quotechar='"', quoting=csv.QUOTE_ALL
        )
        dict_writer.writeheader()

        if options is None:
            options = len(public_keys) * [{}]

        for i, public_key in enumerate(public_keys):
            dict_writer.writerow(dict(options[i], **{'PEM': public_key.pem}))

        return mock_data.getvalue().encode('ascii')

    @staticmethod
    def _get_mock_data_microsoft(public_keys=(), options=None):
        mock_data = io.StringIO()
        dict_writer = csv.DictWriter(
            mock_data, FetcherRootCertificateStoreMicrosoft.CSV_FIELDS,
            quotechar='"', quoting=csv.QUOTE_ALL
        )
        dict_writer.writeheader()

        if options is None:
            options = len(public_keys) * [{}]

        for i, public_key in enumerate(public_keys):
            sha2_256_fingerprint = public_key.fingerprints[Hash.SHA2_256].replace(':', '')
            dict_writer.writerow(dict(options[i], **{'SHA-256 Fingerprint': sha2_256_fingerprint}))

        return mock_data.getvalue().encode('ascii')

    @staticmethod
    def _get_mock_data_apple(public_keys=()):
        mock_data = os.linesep.join([
            '<h2 id="trusted">Trusted Certificates</h2>',
            '<tbody>',
            '<tr>',
        ] + [
            f'<th>{field_name}</th>'
            for field_name in FetcherRootCertificateStoreApple.FIELDS
        ] + [
            '</tr>',
        ])

        for public_key in public_keys:
            mock_data += '<tr>' + os.linesep
            for field_name in FetcherRootCertificateStoreApple.FIELDS:
                if field_name == 'Fingerprint (SHA-256)':
                    data = public_key.fingerprints[Hash.SHA2_256].replace(':', ' ')
                else:
                    data = field_name

                mock_data += f'<td>{data}</td>{os.linesep}'
            mock_data += '</tr>' + os.linesep

        return mock_data.encode('ascii')

    @staticmethod
    def _get_mock_data_google(public_keys=()):
        mock_data = io.BytesIO()

        with tarfile.open(fileobj=mock_data, mode='w:gz') as tar:
            for public_key in public_keys:
                content = public_key.pem.encode('ascii')
                tarinfo = tarfile.TarInfo(public_key.fingerprints[Hash.SHA2_256])
                tarinfo.size = len(content)
                tar.addfile(tarinfo, io.BytesIO(content))

        return mock_data.getvalue()


class TestUpdaterRootCertificateStoreGoogle(TestRootCertificateBase):
    def test_parse_empty(self):
        with mock.patch.object(HttpFetcher, '__call__', return_value=self._get_mock_data_google()):
            root_certificate_store = FetcherRootCertificateStoreGoogle.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 0)

    def test_parse_pem(self):
        public_key_x509 = self._get_public_key_x509('snakeoil_ca_cert')
        mock_data = self._get_mock_data_google([public_key_x509])
        with mock.patch.object(HttpFetcher, '__call__', return_value=mock_data):
            root_certificate_store = FetcherRootCertificateStoreGoogle.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 1)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(public_key_x509.pem.splitlines()): (),
        })


class TestUpdaterRootCertificateStoreApple(TestRootCertificateBase):
    def test_parse_empty(self):
        with mock.patch.object(HttpFetcher, '__call__', return_value=self._get_mock_data_apple()):
            root_certificate_store = FetcherRootCertificateStoreApple.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 0)

    def test_parse_pem(self):
        public_key_x509 = self._get_public_key_x509('snakeoil_ca_cert')
        mock_data_apple = self._get_mock_data_apple([public_key_x509])
        mock_data_crt_sh = public_key_x509.pem.encode('ascii')
        with mock.patch.object(HttpFetcher, '__call__', side_effect=[mock_data_apple, mock_data_crt_sh]):
            root_certificate_store = FetcherRootCertificateStoreApple.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 1)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(public_key_x509.pem.splitlines()): (),
        })


class TestUpdaterRootCertificateStoreMicrosoft(TestRootCertificateBase):
    def test_parse_empty(self):
        mock_data = self._get_mock_data_microsoft()
        with mock.patch.object(HttpFetcher, '__call__', return_value=mock_data):
            root_certificate_store = FetcherRootCertificateStoreMicrosoft.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 0)

    def test_parse_pem(self):
        public_key_x509 = self._get_public_key_x509('snakeoil_ca_cert')

        mock_data_microsoft = self._get_mock_data_microsoft([public_key_x509])
        mock_data_crt_sh = public_key_x509.pem.encode('ascii')

        with mock.patch.object(HttpFetcher, '__call__', side_effect=[mock_data_microsoft, mock_data_crt_sh]):
            root_certificate_store = FetcherRootCertificateStoreMicrosoft.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 1)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(public_key_x509.pem.splitlines()): (),
        })

    def test_parse_status_disabled(self):
        mock_data = self._get_mock_data_microsoft(
            [self.public_key_x509_snakeoil_ca, self.public_key_x509_lets_encrypt],
            [{}, {'Microsoft Status': 'Disabled'}],
        )
        mock_data = [
            mock_data,
            self.public_key_x509_snakeoil_ca.pem.encode('ascii'),
            self.public_key_x509_snakeoil_ca.pem.encode('ascii'),
        ]

        with mock.patch.object(HttpFetcher, '__call__', side_effect=mock_data):
            root_certificate_store = FetcherRootCertificateStoreMicrosoft.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 1)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(self.public_key_x509_snakeoil_ca.pem.splitlines()): (),
        })

    def test_parse_status_not_before(self):
        mock_data = self._get_mock_data_microsoft(
            [self.public_key_x509_snakeoil_ca, self.public_key_x509_lets_encrypt],
            [{}, {'Microsoft Status': 'NotBefore', 'Valid From [GMT]': '1970 Jan 01'}]
        )
        mock_data = [
            mock_data,
            self.public_key_x509_snakeoil_ca.pem.encode('ascii'),
        ]

        with mock.patch.object(HttpFetcher, '__call__', side_effect=mock_data):
            root_certificate_store = FetcherRootCertificateStoreMicrosoft.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 2)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(self.public_key_x509_snakeoil_ca.pem.splitlines()): (),
            tuple(self.public_key_x509_lets_encrypt.pem.splitlines()): (
                CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    date=datetime.datetime(1970, 1, 1, 0, 0),
                ),
            ),
        })


class TestUpdaterRootCertificateStoreMozilla(TestRootCertificateBase):
    def test_parse_empty(self):
        mock_data = self._get_mock_data_mozilla()
        with mock.patch.object(HttpFetcher, '__call__', return_value=mock_data):
            root_certificate_store = FetcherRootCertificateStoreMozilla.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 0)

    def test_parse_pem(self):
        public_key_x509 = self._get_public_key_x509('letsencrypt_isrg_root_x1')

        mock_data = self._get_mock_data_mozilla([public_key_x509])
        with mock.patch.object(HttpFetcher, '__call__', side_effect=[mock_data]):
            root_certificate_store = FetcherRootCertificateStoreMozilla.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 1)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(public_key_x509.pem.splitlines()): (),
        })

    def test_parse_constraint_date(self):
        public_key_x509 = self._get_public_key_x509('letsencrypt_isrg_root_x1')

        mock_data = self._get_mock_data_mozilla(
            [public_key_x509],
            [{'Distrust for TLS After Date': '1970.01.01'}]
        )
        with mock.patch.object(HttpFetcher, '__call__', return_value=mock_data):
            root_certificate_store = FetcherRootCertificateStoreMozilla.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 1)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(public_key_x509.pem.splitlines()): (
                CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    date=datetime.datetime(1970, 1, 1),
                ),
            )
        })

    def test_parse_constraint_domains(self):
        public_key_x509 = self._get_public_key_x509('letsencrypt_isrg_root_x1')

        mock_data = self._get_mock_data_mozilla(
            [public_key_x509],
            [{'Mozilla Applied Constraints': 'domain'}]
        )
        with mock.patch.object(HttpFetcher, '__call__', return_value=mock_data):
            root_certificate_store = FetcherRootCertificateStoreMozilla.from_current_data()
        self.assertEqual(len(root_certificate_store.parsed_data), 1)
        self.assertEqual(root_certificate_store.parsed_data, {
            tuple(public_key_x509.pem.splitlines()): (
                CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    domains=["domain"],
                ),
            )
        })


class UpdaterRootCertificateTrustStoreTest(UpdaterBase):
    def __init__(self):
        root_certificate_test_class = RootCertificateBase(
            'RootCertificate', RootCertificateBase.get_json_records(RootCertificateParams)
        )

        super().__init__(
            fetcher_class=FetcherRootCertificateStore,
            enum_class=root_certificate_test_class,
            enum_param_class=RootCertificateParams,
        )


class TestFetcherRootCertificateStore(TestRootCertificateBase):
    def test_parse_empty(self):
        http_fetcher_results = [
            self._get_mock_data_mozilla(),
            self._get_mock_data_google(),
            self._get_mock_data_microsoft(),
            self._get_mock_data_apple(),
        ]
        with mock.patch.object(HttpFetcher, '__call__', side_effect=http_fetcher_results):
            fetched_data = FetcherRootCertificateStore.from_current_data()

        self.assertEqual(len(fetched_data.parsed_data), 0)

    def test_parse_single_item_in_store(self):
        mock_data_mozilla = self._get_mock_data_mozilla([self.public_key_x509_lets_encrypt])
        mock_data_microsoft = self._get_mock_data_microsoft([self.public_key_x509_snakeoil_ca])
        http_fetcher_results = [
            mock_data_mozilla,
            self._get_mock_data_google(),
            mock_data_microsoft,
            self.public_key_x509_snakeoil_ca.pem.encode('ascii'),
            self._get_mock_data_apple(),
        ]
        with mock.patch.object(HttpFetcher, '__call__', side_effect=http_fetcher_results):
            fetched_data = FetcherRootCertificateStore.from_current_data()

        self.assertEqual(len(fetched_data.parsed_data), 2)
        for root_certificate_param in fetched_data.parsed_data:
            trust_stores = root_certificate_param.trust_stores
            self.assertEqual(len(trust_stores), 1)

    def test_parse_multiple_item_in_store(self):
        mock_data_mozilla = self._get_mock_data_mozilla([
            self.public_key_x509_lets_encrypt,
            self.public_key_x509_snakeoil_ca,
        ])
        mock_data_microsoft = self._get_mock_data_microsoft([
            self.public_key_x509_lets_encrypt,
            self.public_key_x509_snakeoil_ca
        ])
        http_fetcher_results = [
            mock_data_mozilla,
            self._get_mock_data_google(),
            mock_data_microsoft,
            self.public_key_x509_snakeoil_ca.pem.encode('ascii'),
            self._get_mock_data_apple(),
        ]
        with mock.patch.object(HttpFetcher, '__call__', side_effect=http_fetcher_results):
            fetched_data = FetcherRootCertificateStore.from_current_data()

        self.assertEqual(len(fetched_data.parsed_data), 2)
        for root_certificate_param in fetched_data.parsed_data:
            trust_stores = root_certificate_param.trust_stores
            self.assertEqual(len(trust_stores), 2)
        for root_certificate_param in fetched_data.parsed_data:
            trust_stores = root_certificate_param.trust_stores
            for trust_store in trust_stores:
                constraints = trust_store.constraints
                self.assertEqual(len(constraints), 0)


class TestRootCertificateStore(TestRootCertificateBase):
    def test_default(self):
        root_certificate_store = RootCertificateStore()
        self.assertEqual(root_certificate_store.certificates, {})

    def test_with_certificates(self):
        base64_data = Base64Data(b'test data')
        constraint_action = RootCertificateTrustConstraintAction.DISTRUST
        root_certificate_store = RootCertificateStore(
            certificates={base64_data: constraint_action}
        )
        self.assertEqual(root_certificate_store.certificates, {base64_data: constraint_action})

    def test_error_invalid_key_type(self):
        with self.assertRaises(TypeError):
            RootCertificateStore(certificates={'invalid': RootCertificateTrustConstraintAction.DISTRUST})

    def test_error_invalid_value_type(self):
        with self.assertRaises(TypeError):
            RootCertificateStore(certificates={Base64Data(b'test'): 'invalid'})



# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.common.algorithm import BlockCipher
from cryptodatahub.ike.algorithm import (
    Ikev2PayloadType,
    Ikev2NotifyType,
    Ikev2ExtendedSequenceNumber,
    Ikev2TransformType,
    Ikev2AuthenticationMethod,
    Ikev2DiffieHellmanGroup,
    Ikev2EncryptionAlgorithm,
    Ikev2EncryptionAlgorithmParams,
    Ikev2HashAlgorithm,
    Ikev2IntegrityAlgorithm,
    Ikev2PseudorandomFunction,
    Ikev1AttributeType,
    Ikev1AuthenticationMethod,
    Ikev1DiffieHellmanGroup,
    Ikev1Doi,
    Ikev1EncryptionAlgorithm,
    Ikev1EncryptionAlgorithmParams,
    Ikev1GroupType,
    Ikev1HashAlgorithm,
    Ikev1LifeType,
    Ikev1NotifyType,
    Ikev1PayloadType,
    Ikev1ProtocolId,
    Ikev1TransformType,
)


class TestIkev2PseudorandomFunction(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2PseudorandomFunction

    def test_str(self):
        self.assertEqual(str(Ikev2PseudorandomFunction.PRF_HMAC_MD5.value), 'MD5')


class TestIkev2AuthenticationMethod(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2AuthenticationMethod

    def test_str(self):
        self.assertEqual(
            str(Ikev2AuthenticationMethod.RSA_DIGITAL_SIGNATURE.value),
            'RSA Digital Signature'
        )


class TestIkev2DiffieHellmanGroup(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2DiffieHellmanGroup

    def test_str(self):
        self.assertEqual(
            str(Ikev2DiffieHellmanGroup.MODP_GROUP_768_BIT.value),
            '768-bit MODP Group/Oakley Group 1'
        )


class TestIkev2EncryptionAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2EncryptionAlgorithm

    def test_str(self):
        self.assertEqual(
            str(Ikev2EncryptionAlgorithm.ENCR_DES_IV64.value),
            'DES (CBC)'
        )
        self.assertEqual(
            str(Ikev2EncryptionAlgorithmParams(
                code=0,
                bulk_ciphers=[BlockCipher.CHACHA20],
                block_cipher_mode=None,
            )),
            'ChaCha20'
        )
        self.assertEqual(
            str(Ikev2EncryptionAlgorithm.ENCR_NULL.value),
            'null'
        )


class TestIkev2HashAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2HashAlgorithm

    def test_str(self):
        self.assertEqual(
            str(Ikev2HashAlgorithm.HASH_MD5.value),
            'MD5'
        )


class TestIkev2IntegrityAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2IntegrityAlgorithm

    def test_str(self):
        self.assertEqual(
            str(Ikev2IntegrityAlgorithm.AUTH_HMAC_MD5_96.value),
            'MD5'
        )
        self.assertEqual(
            str(Ikev2IntegrityAlgorithm.NONE.value),
            'null'
        )


class TestIkev2PayloadType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2PayloadType


class TestIkev2NotifyType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2NotifyType


class TestIkev2ExtendedSequenceNumber(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2ExtendedSequenceNumber


class TestIkev2TransformType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev2TransformType


class TestIkev1AttributeType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1AttributeType


class TestIkev1Doi(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1Doi


class TestIkev1NotifyType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1NotifyType


class TestIkev1PayloadType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1PayloadType


class TestIkev1ProtocolId(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1ProtocolId


class TestIkev1TransformType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1TransformType


class TestIkev1LifeType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1LifeType


class TestIkev1GroupType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1GroupType


class TestIkev1AuthenticationMethod(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1AuthenticationMethod


class TestIkev1DiffieHellmanGroup(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1DiffieHellmanGroup

    def test_str(self):
        self.assertEqual(
            str(Ikev1DiffieHellmanGroup.MODP_768_BIT.value),
            '768-bit MODP Group/Oakley Group 1'
        )


class TestIkev1EncryptionAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1EncryptionAlgorithm

    def test_str(self):
        self.assertEqual(
            str(Ikev1EncryptionAlgorithm.DES_CBC.value),
            'DES (CBC)'
        )
        self.assertEqual(
            str(Ikev1EncryptionAlgorithmParams(
                code=0,
                bulk_ciphers=[],
                block_cipher_mode=None,
            )),
            'null'
        )
        self.assertEqual(
            str(Ikev1EncryptionAlgorithmParams(
                code=0,
                bulk_ciphers=[BlockCipher.AES_128],
                block_cipher_mode=None,
            )),
            'AES-128'
        )


class TestIkev1HashAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return Ikev1HashAlgorithm

    def test_str(self):
        self.assertEqual(
            str(Ikev1HashAlgorithm.MD5.value),
            'MD5'
        )



# -*- coding: utf-8 -*-

import abc
import base64
import collections
import datetime
import enum
import json
import inspect
import os
import pathlib
import re
import unicodedata

import dateutil.parser

import attr
import urllib3

from cryptodatahub.common.exception import InvalidValue


class _ConverterBase():
    @abc.abstractmethod
    def __call__(self, value):
        raise NotImplementedError()

    @abc.abstractmethod
    def __repr__(self):
        raise NotImplementedError()


@attr.s(repr=False, slots=True, hash=True)
class _DictObjectConverter(_ConverterBase):
    object_type = attr.ib(validator=attr.validators.instance_of(type))

    def __call__(self, value):
        if value is None:
            return None

        try:
            return self.object_type(**value)
        except TypeError:
            pass

        return value

    def __repr__(self):
        return '<dict to object converter>'


def convert_dict_to_object(object_type):
    return _DictObjectConverter(object_type)


@attr.s(repr=False, slots=True, hash=True)
class _ValueToObjectConverter(_ConverterBase):
    object_type = attr.ib(validator=attr.validators.instance_of(type))
    value_converter = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(_ConverterBase)))

    def __call__(self, value):
        if value is None:
            return None

        if isinstance(value, self.object_type):
            return value

        if self.value_converter is not None:
            value = self.value_converter(value)

        return self.object_type(value)

    def __repr__(self):
        return '<value to object converter>'


def convert_value_to_object(object_type, value_converter=None):
    return _ValueToObjectConverter(object_type, value_converter)


@attr.s(repr=False, slots=True, hash=True)
class _EnumConverter(_ConverterBase):
    enum_type = attr.ib(validator=attr.validators.instance_of(type))

    def __call__(self, value):
        if value is None:
            return None

        if not isinstance(value, str):
            return value

        try:
            value = getattr(self.enum_type, value.upper())
        except AttributeError:
            pass

        return value

    def __repr__(self):
        return '<enum converter>'


def convert_enum(enum_type):
    return _EnumConverter(enum_type)


@attr.s(frozen=True)
class Base64Data():
    value = attr.ib(validator=attr.validators.instance_of((bytes, bytearray)))

    def _asdict(self):
        return str(self)

    def __str__(self):
        return base64.b64encode(self.value).decode('ascii')


@attr.s(repr=False, slots=True, hash=True)
class _DateTimeConverter(_ConverterBase):
    strptime_format = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(str)))

    def __call__(self, date_time):
        if date_time is None:
            return None

        if isinstance(date_time, datetime.datetime):
            return date_time

        try:
            if self.strptime_format is None:
                date_time = dateutil.parser.isoparse(date_time)
            else:
                date_time = datetime.datetime.strptime(date_time, self.strptime_format)
        except (TypeError, ValueError):
            pass

        return date_time

    def __repr__(self):
        return '<datetime converter>'


def convert_datetime(strptime_format=None):
    return _DateTimeConverter(strptime_format)


@attr.s(repr=False, slots=True, hash=True)
class _Base64DataConverter(_ConverterBase):
    def __call__(self, value):
        if value is None:
            return None

        if isinstance(value, (bytearray, bytes)):
            return Base64Data(value)

        if not isinstance(value, str):
            return value

        try:
            value = Base64Data(base64.b64decode(value))
        except (ValueError, TypeError):
            pass

        return value

    def __repr__(self):
        return '<base64 data converter>'


def convert_base64_data():
    return _Base64DataConverter()


@attr.s(repr=False, slots=True, hash=True)
class _BigNumberConverter(_ConverterBase):
    def __call__(self, value):
        if value is None:
            return None

        if not isinstance(value, collections.abc.Iterable):
            return value

        if not all(map(lambda big_number_part: isinstance(big_number_part, str), value)):
            return value

        try:
            value = int(''.join(value).replace('0x', '').replace(' ', '').replace(',', ''), 16)
        except ValueError:
            pass

        return value

    def __repr__(self):
        return '<big number converter>'


def convert_big_enum():
    return _BigNumberConverter()


@attr.s(repr=False, slots=True, hash=True)
class _IterableConverter(_ConverterBase):
    member_converter = attr.ib(validator=attr.validators.instance_of(collections.abc.Callable))

    def __call__(self, iterable):
        if iterable is None:
            return None

        try:
            for idx, member in enumerate(iterable):
                iterable[idx] = self.member_converter(member)
        except (TypeError, ValueError):
            pass

        return iterable

    def __repr__(self):
        return '<iterable converter>'


def convert_iterable(member_converter):
    return _IterableConverter(member_converter)


@attr.s(repr=False, slots=True, hash=True)
class _MappingConverter(_ConverterBase):
    key_converter = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(collections.abc.Callable)))
    value_converter = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(collections.abc.Callable)))

    def __call__(self, mapping):
        if mapping is None:
            return None

        if not isinstance(mapping, collections.abc.Mapping):
            return mapping

        try:
            key_value_pairs = [[key, value] for key, value in mapping.items()]
            if self.key_converter is not None:
                for pair in key_value_pairs:
                    pair[0] = self.key_converter(pair[0])
            if self.value_converter is not None:
                for pair in key_value_pairs:
                    pair[1] = self.value_converter(pair[1])
            mapping = type(mapping)(key_value_pairs)
        except (TypeError, ValueError):
            pass

        return mapping

    def __repr__(self):
        return '<mapping converter>'


def convert_mapping(key_converter=None, value_converter=None):
    return _MappingConverter(key_converter, value_converter)


@attr.s(frozen=True)
class ClientVersion():
    parts = attr.ib(validator=attr.validators.deep_iterable(attr.validators.instance_of(int)))

    @classmethod
    def from_str(cls, version_str):
        try:
            return cls((int(version_str), ))
        except ValueError:
            pass

        return cls(tuple(map(int, version_str.split('.'))))

    def __str__(self):
        return '.'.join(map(str, self.parts))


@attr.s(repr=False, slots=True, hash=True)
class _ClientVersionConverter(_ConverterBase):
    def __call__(self, version):
        if version is None:
            return None

        if not isinstance(version, str):
            return version

        try:
            version = ClientVersion.from_str(version)
        except (TypeError, ValueError):
            pass

        return version

    def __repr__(self):
        return '<client version converter>'


def convert_client_version():
    return _ClientVersionConverter()


@attr.s(repr=False, slots=True, hash=True)
class _UrlConverter(_ConverterBase):
    def __call__(self, value):
        if value is None:
            return None

        if not isinstance(value, str):
            return value

        try:
            value = urllib3.util.parse_url(value)
        except urllib3.exceptions.LocationParseError:
            pass

        return value

    def __repr__(self):
        return '<url converter>'


def convert_url():
    return _UrlConverter()


@attr.s(repr=False, slots=True, hash=True)
class _VariadicConverter(_ConverterBase):
    converters = attr.ib(
        validator=attr.validators.deep_iterable(attr.validators.instance_of(_ConverterBase))
    )

    def __call__(self, convertable):
        if convertable is None:
            return None

        for converter in self.converters:
            converted = converter(convertable)
            if id(converted) != id(convertable):
                return converted

        return convertable

    def __repr__(self):
        return '<variadic converter>'


def convert_variadic(converters):
    return _VariadicConverter(converters)


class CryptoDataParamsBase():
    @classmethod
    def get_init_attribute_names(cls):
        return [
            name
            for name, attribute in attr.fields_dict(cls).items()
            if attribute.init
        ]

    @staticmethod
    def _asdict_filter(attribute, value):
        if attribute.name.startswith('_'):
            return False

        if attribute.default != attr.NOTHING and value == attribute.default:
            return False

        return True

    @classmethod
    def _asdict_serializer(cls, _, __, value):
        if hasattr(value, '_asdict'):
            return getattr(value, '_asdict')()
        if isinstance(value, enum.Enum):
            return value.name

        return value

    def _asdict(self):
        return attr.asdict(
            self,
            filter=self._asdict_filter,
            dict_factory=collections.OrderedDict,
            value_serializer=self._asdict_serializer
        )


@attr.s(frozen=True)
class CryptoDataParamsFetchedBase(CryptoDataParamsBase):
    @property
    @abc.abstractmethod
    def identifier(self):
        raise NotImplementedError()


@attr.s(frozen=True)
class CryptoDataParamsNamed(CryptoDataParamsBase):
    name = attr.ib(validator=attr.validators.instance_of(str))
    long_name = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(str)))

    def __str__(self):
        return self.name


@attr.s(frozen=True)
class CryptoDataParamsEnumNumeric(CryptoDataParamsBase):
    code = attr.ib()

    @code.validator
    def _validator_code(self, _, value):
        if not isinstance(value, int):
            raise ValueError(value)
        if value < 0:
            raise ValueError(value)
        if value >= 2 ** (self.get_code_size() * 8):
            raise ValueError(value)

    @classmethod
    def get_code_size(cls):
        raise NotImplementedError()


@attr.s(frozen=True)
class CryptoDataParamsEnumString(CryptoDataParamsBase):
    code = attr.ib(validator=attr.validators.instance_of(str))

    def __str__(self):
        return self.code

    def _asdict(self):
        return str(self)

    def get_code_size(self):
        return len(self.code)


@attr.s(frozen=True)
class CryptoDataParamsOIDOptional(CryptoDataParamsNamed):
    oid = attr.ib(validator=attr.validators.optional(attr.validators.instance_of(str)))


class CryptoDataEnumBase(enum.Enum):
    @classmethod
    def get_json_records(cls, param_class):
        return collections.OrderedDict([
            (unicodedata.normalize('NFD', name).encode('ascii', 'ignore').decode('utf-8'), param_class(**{
                init_param_name.replace('-', '_'): value
                for init_param_name, value in params.items()
                if not init_param_name.startswith('_')
            }))
            for name, params in cls.get_json_object(param_class).items()
        ])

    @staticmethod
    def get_file_name_from_param_class(param_class):
        if not param_class.__name__.endswith('Params'):
            raise TypeError(param_class)

        enum_class_name = param_class.__name__[:-6]
        enum_class_name_parts = [
            name_part.lower()
            for name_part in re.split("([A-Z]+[^A-Z]*)", enum_class_name)
            if name_part
        ]

        return '-'.join(enum_class_name_parts) + '.json'

    @classmethod
    def get_json_path(cls, param_class):
        return (
            pathlib.Path(inspect.getfile(param_class)).parent /
            cls.get_file_name_from_param_class(param_class)
        )

    @classmethod
    def get_json_encoding(cls):
        return 'ascii'

    @classmethod
    def is_json_encoding_ascii(cls):
        return cls.get_json_encoding() == 'ascii'

    @classmethod
    def get_json_object(cls, param_class):
        json_path = cls.get_json_path(param_class)
        with open(str(json_path), 'r', encoding=cls.get_json_encoding()) as json_file:
            return json.load(json_file, object_pairs_hook=collections.OrderedDict)

    @classmethod
    def dump_json(cls, json_object):
        return json.dumps(json_object, ensure_ascii=cls.is_json_encoding_ascii(), indent=4) + os.linesep

    @classmethod
    def set_json(cls, param_class, json_object):
        json_path = cls.get_json_path(param_class)
        with open(str(json_path), 'w+', encoding=cls.get_json_encoding()) as json_file:
            json_file.write(cls.dump_json(json_object))

    @classmethod
    def get_json(cls, param_class):
        return cls.get_json_object(param_class)

    @classmethod
    def _from_attr(cls, attr_name, value):
        for item in cls:
            if value == getattr(item.value, attr_name):
                return item

        raise InvalidValue(value, cls, attr_name)


class CryptoDataEnumCodedBase(CryptoDataEnumBase):
    @classmethod
    def from_code(cls, code):
        return cls._from_attr('code', code)


class CryptoDataEnumOIDBase(CryptoDataEnumCodedBase):
    @classmethod
    def from_oid(cls, oid):
        return cls._from_attr('oid', oid)



# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.common.entity import Entity, EntityRole, Server


class TestEntity(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return Entity

    def test_get_items_by_role(self):
        trust_store_owners = Entity.get_items_by_role(EntityRole.CA_TRUST_STORE_OWNER)
        self.assertEqual(len(trust_store_owners), 4)
        self.assertEqual(id(trust_store_owners), id(Entity.get_items_by_role(EntityRole.CA_TRUST_STORE_OWNER)))


class TestServer(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return Server

    def test_str(self):
        self.assertEqual(str(Server.HAPROXY.value), 'HAProxy')
        self.assertEqual(str(Server.NGINX.value), 'NGINX (F5)')



# -*- coding: utf-8 -*-

import collections

import attr


@attr.s
class UpdaterBase():
    fetcher_class = attr.ib(validator=attr.validators.instance_of(type))
    enum_class = attr.ib(validator=attr.validators.instance_of(type))
    enum_param_class = attr.ib(validator=attr.validators.instance_of(type))

    def _has_item_changed(self, enum_item_name, enum_item_value):
        try:
            return self.enum_class[enum_item_name].value != enum_item_value
        except KeyError:
            return True

    def _has_data_changed(self, current_data_items_by_name):
        return any(map(
            lambda item: self._has_item_changed(*item),
            current_data_items_by_name.items()
        ))

    def __call__(self):
        current_data = self.fetcher_class.from_current_data()
        current_data_items_by_name = {
            fetched_data_item.identifier: fetched_data_item
            for fetched_data_item in current_data.parsed_data
        }
        if self._has_data_changed(current_data_items_by_name):
            self.enum_class.set_json(self.enum_param_class, collections.OrderedDict([
                (identifier, current_data_items_by_name[identifier]._asdict())
                for identifier in sorted(current_data_items_by_name.keys())
            ]))



# -*- coding: utf-8 -*-

import enum
import typing

import attr

from cryptodatahub.common.types import (
    CryptoDataEnumBase,
    CryptoDataEnumCodedBase,
    CryptoDataParamsEnumNumeric,
    convert_enum,
    convert_iterable,
    convert_variadic,
)
from cryptodatahub.common.algorithm import BlockCipher, Hash, MAC, NamedGroup, BlockCipherMode
from cryptodatahub.common.parameter import DHParamWellKnown


@attr.s(frozen=True)
class Ikev2PseudorandomFunctionParams(CryptoDataParamsEnumNumeric):
    """Pseudorandom function parameters."""

    mac: MAC = attr.ib(
        converter=convert_enum(MAC),
        validator=attr.validators.instance_of(MAC)
    )

    def __str__(self):
        return self.mac.value.name

    @classmethod
    def get_code_size(cls):
        return 2


Ikev2PseudorandomFunction = CryptoDataEnumCodedBase(
    'PseudorandomFunction',
    CryptoDataEnumBase.get_json_records(Ikev2PseudorandomFunctionParams)
)


@attr.s(frozen=True)
class Ikev2AuthenticationMethodParams(CryptoDataParamsEnumNumeric):
    """Authentication method parameters."""

    description: str = attr.ib(
        validator=attr.validators.instance_of(str)
    )

    def __str__(self):
        return self.description

    @classmethod
    def get_code_size(cls):
        return 2


Ikev2AuthenticationMethod = CryptoDataEnumCodedBase(
    'AuthenticationMethod',
    CryptoDataEnumBase.get_json_records(Ikev2AuthenticationMethodParams)
)


@attr.s(frozen=True)
class Ikev2IntegrityAlgorithmParams(CryptoDataParamsEnumNumeric):
    """Integrity algorithm parameters."""

    hmac: typing.Optional[MAC] = attr.ib(
        converter=convert_enum(MAC),
        validator=attr.validators.optional(attr.validators.instance_of(MAC))
    )

    def __str__(self):
        if self.hmac is None:
            return "null"

        return self.hmac.value.name

    @classmethod
    def get_code_size(cls):
        return 2


Ikev2IntegrityAlgorithm = CryptoDataEnumCodedBase(
    'IntegrityAlgorithm',
    CryptoDataEnumBase.get_json_records(Ikev2IntegrityAlgorithmParams)
)


@attr.s(frozen=True)
class Ikev2DiffieHellmanGroupParams(CryptoDataParamsEnumNumeric):
    """Diffie-Hellman group parameters."""

    key_parameter: typing.Union[NamedGroup, DHParamWellKnown, str] = attr.ib(
        converter=convert_variadic((convert_enum(NamedGroup), convert_enum(DHParamWellKnown))),
        validator=attr.validators.optional(
            attr.validators.instance_of((NamedGroup, DHParamWellKnown, str))
        )
    )

    def __str__(self):
        return self.key_parameter.value.name

    @classmethod
    def get_code_size(cls):
        return 2


Ikev2DiffieHellmanGroup = CryptoDataEnumCodedBase(
    'DiffieHellmanGroup',
    CryptoDataEnumBase.get_json_records(Ikev2DiffieHellmanGroupParams)
)


@attr.s(frozen=True)
class Ikev2EncryptionAlgorithmParams(CryptoDataParamsEnumNumeric):
    """Encryption algorithm parameters."""

    bulk_ciphers: typing.List[BlockCipher] = attr.ib(
        converter=convert_iterable(convert_enum(BlockCipher)),
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(BlockCipher)),
    )
    block_cipher_mode: typing.Optional[BlockCipherMode] = attr.ib(
        converter=convert_enum(BlockCipherMode),
        validator=attr.validators.optional(attr.validators.instance_of(BlockCipherMode)),
    )

    def __str__(self):
        if not self.bulk_ciphers:
            return 'null'

        cipher_name = self.bulk_ciphers[0].value.name
        if self.block_cipher_mode is None:
            return cipher_name

        return f'{cipher_name} ({self.block_cipher_mode.value.name})'

    @classmethod
    def get_code_size(cls):
        return 2


Ikev2EncryptionAlgorithm = CryptoDataEnumCodedBase(
    'EncryptionAlgorithm',
    CryptoDataEnumBase.get_json_records(Ikev2EncryptionAlgorithmParams)
)


@attr.s(frozen=True)
class Ikev2HashAlgorithmParams(CryptoDataParamsEnumNumeric):
    """Hash algorithm parameters."""

    hash_algorithm: Hash = attr.ib(
        converter=convert_enum(Hash),
        validator=attr.validators.instance_of(Hash)
    )

    def __str__(self):
        return self.hash_algorithm.value.name

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2HashAlgorithm = CryptoDataEnumBase(
    'HashAlgorithm',
    CryptoDataEnumBase.get_json_records(Ikev2HashAlgorithmParams)
)


@attr.s(frozen=True)
class Ikev2ExchangeTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv2 exchange type parameters."""

    description: str = attr.ib(
        validator=attr.validators.instance_of(str)
    )

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2ExchangeType = CryptoDataEnumCodedBase(
    'ExchangeType',
    CryptoDataEnumBase.get_json_records(Ikev2ExchangeTypeParams)
)


@attr.s(frozen=True)
class Ikev1ExchangeTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv1 exchange type parameters."""

    description: str = attr.ib(
        validator=attr.validators.instance_of(str)
    )

    @classmethod
    def get_code_size(cls):
        return 1


Ikev1ExchangeType = CryptoDataEnumCodedBase(
    'ExchangeType',
    CryptoDataEnumBase.get_json_records(Ikev1ExchangeTypeParams)
)


class Ikev2NotifyLevel(enum.Enum):
    """Notify level."""
    ERROR = enum.auto()
    STATUS = enum.auto()


@attr.s(frozen=True)
class Ikev2NotifyTypeParams(CryptoDataParamsEnumNumeric):
    """Notify type parameters."""

    level: Ikev2NotifyLevel = attr.ib(
        converter=convert_enum(Ikev2NotifyLevel),
        validator=attr.validators.instance_of(Ikev2NotifyLevel)
    )

    @classmethod
    def get_code_size(cls):
        return 2


Ikev2NotifyType = CryptoDataEnumCodedBase('NotifyType', CryptoDataEnumBase.get_json_records(Ikev2NotifyTypeParams))


class Ikev2TransformAttributeFormat(enum.Enum):
    """IKEv2 attribute format."""
    TV = enum.auto()
    TLV = enum.auto()


@attr.s(frozen=True)
class Ikev2TransformAttributeTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv2 attribute type parameters."""

    format = attr.ib(
        converter=convert_enum(Ikev2TransformAttributeFormat),
        validator=attr.validators.instance_of(Ikev2TransformAttributeFormat)
    )

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2TransformAttributeType = CryptoDataEnumCodedBase(
    'AttributeType',
    CryptoDataEnumBase.get_json_records(Ikev2TransformAttributeTypeParams)
)


@attr.s(frozen=True)
class Ikev2ExtendedSequenceNumberParams(CryptoDataParamsEnumNumeric):
    """Extended sequence number parameters."""

    description: str = attr.ib(
        validator=attr.validators.instance_of(str)
    )

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2ExtendedSequenceNumber = CryptoDataEnumBase(
    'ExtendedSequenceNumber',
    CryptoDataEnumBase.get_json_records(Ikev2ExtendedSequenceNumberParams)
)


@attr.s(frozen=True)
class Ikev2IpCompTransformIdParams(CryptoDataParamsEnumNumeric):
    """IP compression transform ID parameters."""

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2IpCompTransformId = CryptoDataEnumBase(
    'IpCompTransformId',
    CryptoDataEnumBase.get_json_records(Ikev2IpCompTransformIdParams)
)


@attr.s(frozen=True)
class Ikev2TransformTypeParams(CryptoDataParamsEnumNumeric):
    """Transform type parameters."""

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2TransformType = CryptoDataEnumCodedBase(
    'TransformType',
    CryptoDataEnumBase.get_json_records(Ikev2TransformTypeParams)
)


@attr.s(frozen=True)
class Ikev2ProtocolIdParams(CryptoDataParamsEnumNumeric):
    """Protocol ID parameters."""

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2ProtocolId = CryptoDataEnumCodedBase(
    'ProtocolId',
    CryptoDataEnumBase.get_json_records(Ikev2ProtocolIdParams)
)


@attr.s
class Ikev2PayloadTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv2 payload type parameters."""

    description: str = attr.ib(validator=attr.validators.instance_of(str))

    @classmethod
    def get_code_size(cls):
        return 1


Ikev2PayloadType = CryptoDataEnumCodedBase(
    'PayloadType',
    CryptoDataEnumBase.get_json_records(Ikev2PayloadTypeParams)
)


@attr.s(frozen=True)
class Ikev1DoiParams(CryptoDataParamsEnumNumeric):
    """IKEv1 DOI type parameters."""

    @classmethod
    def get_code_size(cls):
        return 4


Ikev1Doi = CryptoDataEnumCodedBase(
    'Doi',
    CryptoDataEnumBase.get_json_records(Ikev1DoiParams)
)


class Ikev1NotifyLevel(enum.Enum):
    """IKEv1 notify level."""
    ERROR = enum.auto()
    STATUS = enum.auto()


@attr.s(frozen=True)
class Ikev1NotifyTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv1 notify type parameters."""

    level: Ikev1NotifyLevel = attr.ib(
        converter=convert_enum(Ikev1NotifyLevel),
        validator=attr.validators.instance_of(Ikev1NotifyLevel)
    )

    @classmethod
    def get_code_size(cls):
        return 2


Ikev1NotifyType = CryptoDataEnumCodedBase(
    'NotifyType',
    CryptoDataEnumBase.get_json_records(Ikev1NotifyTypeParams)
)


class Ikev1AttributeFormat(enum.Enum):
    """IKEv1 attribute format."""
    BASIC = enum.auto()  # B
    VARIABLE = enum.auto()  # V


@attr.s(frozen=True)
class Ikev1AttributeTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv1 attribute type parameters."""

    format: Ikev1AttributeFormat = attr.ib(
        converter=convert_enum(Ikev1AttributeFormat),
        validator=attr.validators.instance_of(Ikev1AttributeFormat)
    )

    @classmethod
    def get_code_size(cls):
        return 1


Ikev1AttributeType = CryptoDataEnumCodedBase(
    'AttributeType',
    CryptoDataEnumBase.get_json_records(Ikev1AttributeTypeParams)
)


@attr.s(frozen=True)
class Ikev1TransformIdParams(CryptoDataParamsEnumNumeric):
    """IKEv1 transform ID parameters."""

    description: str = attr.ib(
        validator=attr.validators.instance_of(str)
    )

    @classmethod
    def get_code_size(cls):
        return 1


Ikev1TransformId = CryptoDataEnumCodedBase(
    'TransformId',
    CryptoDataEnumBase.get_json_records(Ikev1TransformIdParams)
)


@attr.s(frozen=True)
class Ikev1ProtocolIdParams(CryptoDataParamsEnumNumeric):
    """IKEv1 protocol ID parameters."""

    @classmethod
    def get_code_size(cls):
        return 1


Ikev1ProtocolId = CryptoDataEnumCodedBase(
    'ProtocolId',
    CryptoDataEnumBase.get_json_records(Ikev1ProtocolIdParams)
)


@attr.s(frozen=True)
class Ikev1TransformTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv1 transform type parameters."""

    @classmethod
    def get_code_size(cls):
        return 1


Ikev1TransformType = CryptoDataEnumCodedBase(
    'TransformType',
    CryptoDataEnumBase.get_json_records(Ikev1TransformTypeParams)
)


@attr.s(frozen=True)
class Ikev1PayloadTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv1 payload type parameters."""

    description: str = attr.ib(validator=attr.validators.instance_of(str))

    @classmethod
    def get_code_size(cls):
        return 1


Ikev1PayloadType = CryptoDataEnumCodedBase(
    'PayloadType',
    CryptoDataEnumBase.get_json_records(Ikev1PayloadTypeParams)
)


@attr.s(frozen=True)
class Ikev1EncryptionAlgorithmParams(CryptoDataParamsEnumNumeric):
    """IKEv1 encryption algorithm parameters."""

    bulk_ciphers: typing.List[BlockCipher] = attr.ib(
        converter=convert_iterable(convert_enum(BlockCipher)),
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(BlockCipher)),
    )
    block_cipher_mode: typing.Optional[BlockCipherMode] = attr.ib(
        converter=convert_enum(BlockCipherMode),
        validator=attr.validators.optional(attr.validators.instance_of(BlockCipherMode)),
    )

    def __str__(self):
        if not self.bulk_ciphers:
            return 'null'

        cipher_name = self.bulk_ciphers[0].value.name
        if self.block_cipher_mode is None:
            return cipher_name

        return f'{cipher_name} ({self.block_cipher_mode.value.name})'

    @classmethod
    def get_code_size(cls):
        return 2


Ikev1EncryptionAlgorithm = CryptoDataEnumCodedBase(
    'EncryptionAlgorithm',
    CryptoDataEnumBase.get_json_records(Ikev1EncryptionAlgorithmParams)
)


@attr.s(frozen=True)
class Ikev1HashAlgorithmParams(CryptoDataParamsEnumNumeric):
    """IKEv1 hash algorithm parameters."""

    hash: Hash = attr.ib(
        converter=convert_enum(Hash),
        validator=attr.validators.instance_of(Hash)
    )

    def __str__(self):
        return self.hash.value.name

    @classmethod
    def get_code_size(cls):
        return 2


Ikev1HashAlgorithm = CryptoDataEnumCodedBase(
    'Ikev1HashAlgorithm',
    CryptoDataEnumBase.get_json_records(Ikev1HashAlgorithmParams)
)


@attr.s(frozen=True)
class Ikev1AuthenticationMethodParams(CryptoDataParamsEnumNumeric):
    """IKEv1 authentication method parameters."""

    @classmethod
    def get_code_size(cls):
        return 2


Ikev1AuthenticationMethod = CryptoDataEnumCodedBase(
    'Ikev1AuthenticationMethod',
    CryptoDataEnumBase.get_json_records(Ikev1AuthenticationMethodParams)
)


@attr.s(frozen=True)
class Ikev1GroupTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv1 group type parameters."""

    description: str = attr.ib(
        validator=attr.validators.instance_of(str)
    )

    @classmethod
    def get_code_size(cls):
        return 2


Ikev1GroupType = CryptoDataEnumCodedBase(
    'GroupType',
    CryptoDataEnumBase.get_json_records(Ikev1GroupTypeParams)
)


@attr.s(frozen=True)
class Ikev1LifeTypeParams(CryptoDataParamsEnumNumeric):
    """IKEv1 life type parameters."""

    @classmethod
    def get_code_size(cls):
        return 2


Ikev1LifeType = CryptoDataEnumCodedBase(
    'LifeType',
    CryptoDataEnumBase.get_json_records(Ikev1LifeTypeParams)
)


@attr.s(frozen=True)
class Ikev1DiffieHellmanGroupParams(CryptoDataParamsEnumNumeric):
    """IKEv1 Diffie-Hellman group parameters."""

    key_parameter: typing.Union[NamedGroup, DHParamWellKnown, str] = attr.ib(
        converter=convert_variadic((convert_enum(NamedGroup), convert_enum(DHParamWellKnown))),
        validator=attr.validators.instance_of((NamedGroup, DHParamWellKnown, str))
    )

    def __str__(self):
        return self.key_parameter.value.name

    @classmethod
    def get_code_size(cls):
        return 2


Ikev1DiffieHellmanGroup = CryptoDataEnumCodedBase(
    'DiffieHellmanGroup',
    CryptoDataEnumBase.get_json_records(Ikev1DiffieHellmanGroupParams)
)



# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.dnsrec.algorithm import DnsRrType, DnsSecAlgorithm, DnsSecDigestType


class TestDnsRrType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return DnsRrType

    def test_name(self):
        self.assertEqual(str(DnsRrType.CAA.value), 'CAA')


class TestDnsSecAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return DnsSecAlgorithm

    def test_name(self):
        self.assertEqual(str(DnsSecAlgorithm.RSAMD5.value), 'RSA/MD5')


class TestDnsSecDigestType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return DnsSecDigestType

    def test_name(self):
        self.assertEqual(str(DnsSecDigestType.SHA_256.value), 'SHA-256')



# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.tls.client import TlsClient


class TestTlsClient(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return TlsClient

    def test_asdict(self):
        client_dict = TlsClient.FIREFOX_56.value.capabilities._asdict()
        self.assertEqual(
            list(filter(lambda extension_param: extension_param is None, client_dict['extension_params'].values())),
            []
        )

    def test_str(self):
        self.assertEqual(str(TlsClient.FIREFOX_56.value.meta), 'Mozilla Firefox (56 - 72)')



# -*- coding: utf-8 -*-

import collections
import datetime
import os
import unittest
from unittest import mock

from test.common.classes import TestClasses

import asn1crypto

from cryptodatahub.common.algorithm import Authentication, Hash, KeyExchange, NamedGroup, Signature
from cryptodatahub.common.key import (
    _PublicKeySizeGradeable,
    PublicKey,
    PublicKeyParamsDsa,
    PublicKeyParamsEddsa,
    PublicKeyParamsEcdsa,
    PublicKeyParamsRsa,
    PublicKeySize,
    PublicKeyX509Base,
    convert_public_key_size,
)
from cryptodatahub.common.grade import AttackNamed, AttackType, Grade, Vulnerability
from cryptodatahub.common.utils import bytes_from_hex_string

from cryptodatahub.tls.algorithm import TlsExtensionType


class TestPublicKeyConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = '1234'
        converted_value = convert_public_key_size(KeyExchange.DHE)(original_value)
        self.assertEqual(id(original_value), id(convert_public_key_size(KeyExchange.DHE)(converted_value)))

    def test_error_invalid_value(self):
        original_value = -1
        converted_value = convert_public_key_size(KeyExchange.DHE)(original_value)
        self.assertEqual(id(converted_value), id(convert_public_key_size(KeyExchange.DHE)(converted_value)))

    def test_none(self):
        converted_value = convert_public_key_size(KeyExchange.DHE)(None)
        self.assertEqual(converted_value, None)

    def test_convert(self):
        original_value = 2048
        converted_value = convert_public_key_size(KeyExchange.DHE)(original_value)

        self.assertEqual(converted_value, PublicKeySize(KeyExchange.DHE, 2048))

    def test_repr(self):
        self.assertEqual(repr(convert_public_key_size(KeyExchange.DHE)), '<public key size converter>')


class TestPublicKeySizeGradeable(unittest.TestCase):
    def test_gradeable(self):
        self.assertTrue(len(_PublicKeySizeGradeable.get_gradeable_name()))


class TestPublicKeySize(unittest.TestCase):
    @staticmethod
    def _get_vulnerability_values(key_type, key_size):
        return list(PublicKeySize(key_type, key_size).gradeables)

    def test_vulnerabilities(self):
        vulnerability_integer_factorization_weak = Vulnerability(
            attack_type=AttackType.INTEGER_FACTORIZATION,
            grade=Grade.WEAK,
            named=None
        )
        vulnerability_discrete_logarithm_insecure = Vulnerability(
            attack_type=AttackType.DISCRETE_LOGARITHM,
            grade=Grade.INSECURE,
            named=None
        )
        vulnerability_discrete_logarithm_weak = Vulnerability(
            attack_type=AttackType.DISCRETE_LOGARITHM,
            grade=Grade.WEAK,
            named=None
        )
        vulnerability_dheat = Vulnerability(
            attack_type=AttackType.DOS_ATTACK,
            grade=Grade.WEAK,
            named=AttackNamed.DHEAT_ATTACK
        )

        self.assertEqual(PublicKeySize(Authentication.RSA, 2048).gradeables, [_PublicKeySizeGradeable([])])
        self.assertEqual(
            self._get_vulnerability_values(Authentication.RSA, 1024),
            [_PublicKeySizeGradeable([vulnerability_integer_factorization_weak])]
        )
        self.assertEqual(
            self._get_vulnerability_values(Authentication.RSA, 768), [_PublicKeySizeGradeable([Vulnerability(
                attack_type=AttackType.INTEGER_FACTORIZATION,
                grade=Grade.INSECURE,
                named=AttackNamed.FREAK
            )])]
        )

        self.assertEqual(PublicKeySize(Authentication.DSS, 2048).gradeables, [_PublicKeySizeGradeable([])])
        self.assertEqual(
            self._get_vulnerability_values(Authentication.DSS, 1024),
            [_PublicKeySizeGradeable([vulnerability_integer_factorization_weak])]
        )
        self.assertEqual(
            self._get_vulnerability_values(Authentication.DSS, 768), [_PublicKeySizeGradeable([Vulnerability(
                attack_type=AttackType.INTEGER_FACTORIZATION,
                grade=Grade.INSECURE,
                named=None
            )])]

        )

        self.assertEqual(PublicKeySize(KeyExchange.DHE, 2048).gradeables, [_PublicKeySizeGradeable([])])
        self.assertEqual(
            self._get_vulnerability_values(KeyExchange.DHE, 1024),
            [_PublicKeySizeGradeable([vulnerability_integer_factorization_weak])]
        )
        self.assertEqual(
            self._get_vulnerability_values(KeyExchange.DHE, 768), [_PublicKeySizeGradeable([Vulnerability(
                attack_type=AttackType.INTEGER_FACTORIZATION,
                grade=Grade.INSECURE,
                named=AttackNamed.WEAK_DH
            )])]
        )
        self.assertEqual(
            self._get_vulnerability_values(KeyExchange.DHE, 6144),
            [_PublicKeySizeGradeable([vulnerability_dheat])]
        )

        self.assertEqual(PublicKeySize(Authentication.ECDSA, 256).gradeables, [_PublicKeySizeGradeable([])])
        self.assertEqual(
            self._get_vulnerability_values(Authentication.ECDSA, 160),
            [_PublicKeySizeGradeable([vulnerability_discrete_logarithm_weak])]
        )
        self.assertEqual(
            self._get_vulnerability_values(Authentication.ECDSA, 112),
            [_PublicKeySizeGradeable([vulnerability_discrete_logarithm_insecure])]
        )

        self.assertEqual(PublicKeySize(Authentication.EDDSA, 256).gradeables, [_PublicKeySizeGradeable([])])
        self.assertEqual(
            self._get_vulnerability_values(Authentication.EDDSA, 160),
            [_PublicKeySizeGradeable([vulnerability_discrete_logarithm_weak])]
        )
        self.assertEqual(
            self._get_vulnerability_values(Authentication.EDDSA, 112),
            [_PublicKeySizeGradeable([vulnerability_discrete_logarithm_insecure])]
        )

        self.assertEqual(PublicKeySize(Authentication.PSK, 2048).gradeables, None)

    def test_str(self):
        self.assertEqual(str(PublicKeySize(Authentication.RSA, 1024)), '1024')
        self.assertEqual(str(PublicKeySize(Authentication.EDDSA, 256)), '256')


class TestPublicKeyParamsEcdsa(TestClasses.TestKeyBase):
    def test_from_octet_bit_string(self):
        public_key = self._get_public_key('gitlab.com_ssh_ecdsa_key')
        public_key_params = public_key.params

        self.assertEqual(
            public_key_params,
            PublicKeyParamsEcdsa.from_octet_bit_string(
                public_key_params.named_group,
                public_key_params.octet_bit_string,
            )
        )


class TestPublicKey(TestClasses.TestKeyBase):
    def test_from_params(self):
        public_key = self._get_public_key('gitlab.com_ssh_dsa_key')
        public_key_params = public_key.params
        params = PublicKeyParamsDsa(
            prime=public_key_params.prime,
            generator=public_key_params.generator,
            order=public_key_params.order,
            public_key_value=public_key_params.public_key_value,
        )
        public_key = PublicKey.from_params(params)
        self.assertEqual(params, public_key.params)
        self.assertEqual(public_key.key_size, 1024)

        public_key = self._get_public_key('gitlab.com_ssh_ecdsa_key')
        public_key_params = public_key.params
        params = PublicKeyParamsEcdsa(
            named_group=NamedGroup.PRIME256V1,
            point_x=public_key_params.point_x,
            point_y=public_key_params.point_y,
        )
        public_key = PublicKey.from_params(params)
        self.assertEqual(params, public_key.params)
        self.assertEqual(public_key.key_size, 256)

        public_key = self._get_public_key('gitlab.com_ssh_eddsa_key')
        public_key_params = public_key.params
        params = PublicKeyParamsEddsa(
            curve_type=public_key_params.curve_type,
            key_data=public_key_params.key_data,
        )
        public_key = PublicKey.from_params(params)
        self.assertEqual(params, public_key.params)
        self.assertEqual(public_key.key_size, 256)

        public_key = self._get_public_key('ssh_ed448_key')
        public_key_params = public_key.params
        params = PublicKeyParamsEddsa(
            curve_type=public_key_params.curve_type,
            key_data=public_key_params.key_data,
        )
        public_key = PublicKey.from_params(params)
        self.assertEqual(params, public_key.params)
        self.assertEqual(public_key.key_size, 448)

        public_key = self._get_public_key('gitlab.com_ssh_rsa_key')
        public_key_params = public_key.params
        params = PublicKeyParamsRsa(
            modulus=public_key_params.modulus,
            public_exponent=public_key_params.public_exponent
        )
        public_key = PublicKey.from_params(params)
        self.assertEqual(params, public_key.params)
        self.assertEqual(public_key.key_size, 2048)

    def test_pem(self):
        public_key = self._get_public_key('snakeoil_cert_pubkey')
        self.assertEqual(public_key.pem, self._get_public_key_pem('snakeoil_cert_pubkey'))

    def test_der(self):
        public_key = self._get_public_key('snakeoil_cert_pubkey')
        self.assertEqual(public_key, PublicKey.from_der(public_key.der))

    def test_digest(self):
        public_key = self._get_public_key('snakeoil_cert_pubkey')
        self.assertEqual(
            public_key.get_digest(Hash.MD5),
            bytes_from_hex_string('5D:B2:D9:9F:97:5C:C6:19:B3:91:7E:F8:1A:37:2C:78', ':')
        )

    def test_fingerprints(self):
        public_key = self._get_public_key('snakeoil_cert_pubkey')
        self.assertEqual(
            public_key.fingerprints,
            {
                Hash.MD5:
                    '5D:B2:D9:9F:97:5C:C6:19:B3:91:7E:F8:1A:37:2C:78',
                Hash.SHA1:
                    '49:4A:82:89:7C:62:18:44:CF:02:A9:74:E9:57:CD:FC:A2:A4:45:9B',
                Hash.SHA2_256:
                    '29:D4:FC:11:59:74:C3:47:25:25:E7:11:77:44:7C:7A:' +
                    'D3:68:A0:79:70:E5:2D:58:1E:54:7A:60:F9:EA:AF:F4',
            }
        )

    def test_key_type(self):
        public_key_x509 = self._get_public_key_x509('ecc256.badssl.com')
        self.assertEqual(public_key_x509.key_type, Authentication.ECDSA)
        public_key_x509 = self._get_public_key_x509('ecc384.badssl.com')
        self.assertEqual(public_key_x509.key_type, Authentication.ECDSA)

        public_key_x509 = self._get_public_key_x509('rsa2048.badssl.com')
        self.assertEqual(public_key_x509.key_type, Authentication.RSA)
        public_key_x509 = self._get_public_key_x509('rsa4096.badssl.com')
        self.assertEqual(public_key_x509.key_type, Authentication.RSA)
        public_key_x509 = self._get_public_key_x509('rsa8192.badssl.com')
        self.assertEqual(public_key_x509.key_type, Authentication.RSA)

    def test_key_size(self):
        public_key_x509 = self._get_public_key_x509('ecc256.badssl.com')
        self.assertEqual(public_key_x509.key_size, 256)
        public_key_x509 = self._get_public_key_x509('ecc384.badssl.com')
        self.assertEqual(public_key_x509.key_size, 384)

        public_key_x509 = self._get_public_key_x509('rsa2048.badssl.com')
        self.assertEqual(public_key_x509.key_size, 2048)
        public_key_x509 = self._get_public_key_x509('rsa4096.badssl.com')
        self.assertEqual(public_key_x509.key_size, 4096)
        public_key_x509 = self._get_public_key_x509('rsa8192.badssl.com')
        self.assertEqual(public_key_x509.key_size, 8192)

        self.assertEqual(self._get_public_key_x509('gost_2001_cert').key_size, 256)
        self.assertEqual(self._get_public_key_x509('gost_2012_256_cert').key_size, 256)
        self.assertEqual(self._get_public_key_x509('gost_2012_512_cert').key_size, 512)

    def test_asdict(self):
        public_key_x509 = self._get_public_key('gitlab.com_ssh_rsa_key')
        self.assertEqual(public_key_x509._asdict(), collections.OrderedDict([
            ('algorithm', Authentication.RSA),
            ('size', PublicKeySize(key_type=Authentication.RSA, value=2048)),
            ('fingerprints', collections.OrderedDict([
                (Hash.MD5, '94:33:7E:9C:56:1B:CE:19:FF:D3:50:DA:D4:AA:45:D7'),
                (Hash.SHA1, 'BC:14:22:BF:0B:65:89:FB:9E:1C:95:C3:B6:5C:2A:16:1D:82:DB:AE'),
                (
                    Hash.SHA2_256,
                    '34:8E:31:C3:D5:88:1E:09:A4:A9:30:8A:90:D9:46:5B:73:07:79:DE:DA:D2:D8:F2:C3:81:F2:54:24:6B:73:D7'
                )
            ]))
        ]))


class TestPublicKeyX509(TestClasses.TestKeyBase):  # pylint: disable=too-many-public-methods
    def test_eq(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(public_key_x509, public_key_x509)

        self.assertNotEqual(public_key_x509, self._get_public_key_x509('no-common-name.badssl.com'))

        public_key_x509 = self._get_public_key_x509('snakeoil_cert')
        public_key = self._get_public_key('snakeoil_cert_pubkey')
        self.assertEqual(public_key_x509.public_key, public_key)

    def test_pem_lines(self):
        self.assertEqual(
            PublicKeyX509Base.from_pem_lines(self._get_public_key_pem('expired.badssl.com').split(os.linesep)),
            self._get_public_key_x509('expired.badssl.com')
        )

    def test_pem(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(public_key_x509.pem, self._get_public_key_pem('expired.badssl.com'))

    def test_der(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(public_key_x509, PublicKeyX509Base.from_der(public_key_x509.der))

    def test_digest(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(
            public_key_x509.get_digest(Hash.MD5),
            bytes_from_hex_string('67:34:4E:61:C0:43:1C:F1:F7:25:7C:1D:6D:E7:A7:85', ':')
        )

    def test_fingerprints(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(
            public_key_x509.fingerprints,
            {
                Hash.MD5:
                    '67:34:4E:61:C0:43:1C:F1:F7:25:7C:1D:6D:E7:A7:85',
                Hash.SHA1:
                    '40:4B:BD:2F:1F:4C:C2:FD:EE:F1:3A:AB:DD:52:3E:F6:1F:1C:71:F3',
                Hash.SHA2_256:
                    'BA:10:5C:E0:2B:AC:76:88:8E:CE:E4:7C:D4:EB:79:41:' +
                    '65:3E:9A:C9:93:B6:1B:2E:B3:DC:C8:20:14:D2:1B:4F',
            }
        )

    def test_common_name(self):
        public_key_x509 = self._get_public_key_x509('no-common-name.badssl.com')
        self.assertNotEqual(public_key_x509.subject, collections.OrderedDict([]))
        self.assertEqual(public_key_x509.subject_alternative_names, ['no-common-name.badssl.com', ])
        self.assertEqual(public_key_x509.valid_domains, ['no-common-name.badssl.com', ])

    def test_subject_alternative_names(self):
        public_key_x509 = self._get_public_key_x509('no-subject.badssl.com')
        self.assertEqual(public_key_x509.subject, collections.OrderedDict([]))
        self.assertEqual(public_key_x509.subject_alternative_names, ['no-subject.badssl.com'])
        self.assertEqual(public_key_x509.valid_domains, ['no-subject.badssl.com'])

        public_key_x509 = self._get_public_key_x509('badssl.com')
        self.assertNotEqual(public_key_x509.subject, collections.OrderedDict([]))
        self.assertEqual(public_key_x509.subject_alternative_names, ['*.badssl.com', 'badssl.com'])
        self.assertEqual(public_key_x509.valid_domains, ['*.badssl.com', 'badssl.com'])

        public_key_x509 = self._get_public_key_x509('gost_2001_cert')
        self.assertNotEqual(public_key_x509.subject, collections.OrderedDict([]))
        self.assertEqual(public_key_x509.subject_alternative_names, [])

    def test_no_subject(self):
        public_key_x509 = self._get_public_key_x509('no-subject.badssl.com')
        self.assertEqual(public_key_x509.subject_alternative_names, ['no-subject.badssl.com', ])
        self.assertEqual(public_key_x509.valid_domains, ['no-subject.badssl.com', ])

    def test_issuer(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(
            public_key_x509.issuer,
            collections.OrderedDict([
                ('country_name', 'GB'),
                ('state_or_province_name', 'Greater Manchester'),
                ('locality_name', 'Salford'),
                ('organization_name', 'COMODO CA Limited'),
                ('common_name', 'COMODO RSA Domain Validation Secure Server CA')
            ])
        )

    def test_crl_distribution_points(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(
            public_key_x509.crl_distribution_points,
            ['http://crl.comodoca.com/COMODORSADomainValidationSecureServerCA.crl']
        )

        public_key_x509 = self._get_public_key_x509('letsencrypt.org')
        self.assertEqual(
            public_key_x509.crl_distribution_points,
            []
        )

    def test_crl_distribution_points_relative_name(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(
            public_key_x509.crl_distribution_points,
            ['http://crl.comodoca.com/COMODORSADomainValidationSecureServerCA.crl', ]
        )

    def test_ocsp_responders(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(public_key_x509.ocsp_responders, ['http://ocsp.comodoca.com'])

    @mock.patch.object(
        asn1crypto.x509.Certificate, 'authority_information_access_value',
        return_value=None
    )
    def test_ocsp_responders_no_extension(self, _):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(public_key_x509.ocsp_responders, [])

    def test_is_ca(self):
        public_key_x509 = self._get_public_key_x509('letsencrypt_isrg_root_x1')
        self.assertTrue(public_key_x509.is_ca)

        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertFalse(public_key_x509.is_ca)

    def test_self_signed(self):
        public_key_x509 = self._get_public_key_x509('letsencrypt_isrg_root_x1')
        self.assertTrue(public_key_x509.is_self_signed)

        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertFalse(public_key_x509.is_self_signed)

    def test_validity(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertTrue(public_key_x509.expired)
        self.assertEqual(
            public_key_x509.valid_not_before,
            datetime.datetime(2015, 4, 9, 0, 0, tzinfo=asn1crypto.util.timezone.utc)
        )
        self.assertEqual(
            public_key_x509.valid_not_after,
            datetime.datetime(2015, 4, 12, 23, 59, 59, tzinfo=asn1crypto.util.timezone.utc)
        )
        self.assertEqual(
            public_key_x509.validity_period,
            datetime.timedelta(days=4, seconds=-1)
        )
        self.assertEqual(
            public_key_x509.validity_remaining_time,
            None
        )

        public_key_x509 = self._get_public_key_x509('badssl.com')
        self.assertFalse(public_key_x509.expired)

    def test_public_key_pin(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertEqual(
            public_key_x509.public_key_pin,
            '9SLklscvzMYj8f+52lp5ze/hY0CFHyLSPQzSpYYIBm8='
        )

    def test_extended_validation(self):
        public_key_x509 = self._get_public_key_x509('extended-validation.badssl.com')
        self.assertTrue(public_key_x509.extended_validation)

        public_key_x509 = self._get_public_key_x509('badssl.com')
        self.assertFalse(public_key_x509.extended_validation)

        with mock.patch(
            'asn1crypto.x509.Certificate.certificate_policies_value',
            new_callable=mock.PropertyMock
        ) as prop_mock:
            prop_mock.return_value = None
            self.assertFalse(public_key_x509.extended_validation)

    def test_tls_features(self):
        public_key_x509 = self._get_public_key_x509('badssl.com')
        self.assertEqual(public_key_x509.tls_features, [])

        public_key_x509 = self._get_public_key_x509('bitnami.com')
        self.assertEqual(public_key_x509.tls_features, [TlsExtensionType.STATUS_REQUEST])

    def test_signature_algorithm_unknown(self):
        public_key_x509 = self._get_public_key_x509('sha1-intermediate.badssl.com')
        with mock.patch('asn1crypto.algos.SignedDigestAlgorithmId.dotted', new_callable=mock.PropertyMock) as prop_mock:
            prop_mock.side_effect = KeyError('1.2.840.113549.1.1.2')
            self.assertEqual(
                public_key_x509.signature_hash_algorithm,
                Signature.RSA_WITH_MD2
            )

    def test_signature_algorithm(self):
        public_key_x509 = self._get_public_key_x509('comodo_ssl_ca_sha1')
        self.assertEqual(
            public_key_x509.signature_hash_algorithm,
            Signature.RSA_WITH_SHA1
        )

        public_key_x509 = self._get_public_key_x509('sha256.badssl.com')
        self.assertEqual(
            public_key_x509.signature_hash_algorithm,
            Signature.RSA_WITH_SHA2_256
        )
        public_key_x509 = self._get_public_key_x509('sha384.badssl.com')
        self.assertEqual(
            public_key_x509.signature_hash_algorithm,
            Signature.RSA_WITH_SHA2_384
        )
        public_key_x509 = self._get_public_key_x509('sha512.badssl.com')
        self.assertEqual(public_key_x509.signature_hash_algorithm, Signature.RSA_WITH_SHA2_512)

    def test_subject_match(self):
        public_key_x509 = self._get_public_key_x509('expired.badssl.com')
        self.assertTrue(public_key_x509.is_subject_matches('expired.badssl.com'))
        self.assertTrue(public_key_x509.is_subject_matches('whatever.badssl.com'))
        self.assertFalse(public_key_x509.is_subject_matches('what.ever.badssl.com'))

    def test_asdict(self):
        public_key_x509 = self._get_public_key_x509('rsa2048.badssl.com')
        self.assertEqual(public_key_x509._asdict(), collections.OrderedDict([
            ('algorithm', Authentication.RSA),
            ('size', PublicKeySize(
                key_type=Authentication.RSA,
                value=2048,
            )),
            ('version', 'v3'),
            ('serial_number', '398674370847804942957222553262670217215220'),
            ('subject', collections.OrderedDict([('common_name', '*.badssl.com')])),
            ('subject_alternative_names', ['*.badssl.com', 'badssl.com']),
            ('issuer', collections.OrderedDict([
                ('country_name', 'US'),
                ('organization_name', "Let's Encrypt"),
                ('common_name', 'R3')
            ])),
            ('signature_hash_algorithm', Signature.RSA_WITH_SHA2_256),
            ('validity', collections.OrderedDict([
                ('not_before', '2023-04-23 23:00:10+00:00'),
                ('not_after', '2023-07-22 23:00:09+00:00'),
                ('period', '89 days, 23:59:59'),
                ('remaining', None)
            ])),
            ('revocation', collections.OrderedDict([
                ('crl_distribution_points', []),
                ('ocsp_responders', ['http://r3.o.lencr.org'])
            ])),
            ('fingerprints', collections.OrderedDict([
                (Hash.MD5, 'E5:CE:F4:ED:BB:42:C4:59:01:63:29:4B:DD:55:64:E2'),
                (Hash.SHA1, 'B8:68:EC:8E:5C:9F:C2:EC:B2:E5:A7:12:C5:B8:F2:34:B7:33:CD:44'),
                (
                    Hash.SHA2_256,
                    '17:7D:74:6A:01:B5:8C:CD:E4:5D:F9:09:48:E4:01:72:4D:83:6A:C6:E1:44:7F:11:2F:7E:20:C3:40:C0:2A:63'
                )
            ])),
            ('public_key_pin', '6Lkip0FxqykIPcMKjlwSSxNYRqG1EHcSNLMvN4uV1zc='),
            ('end_entity', collections.OrderedDict([
                ('extended_validation', False),
                ('tls_features', [])
            ]))
        ]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from cryptodatahub.common.stores import CertificateTransparencyLog, CertificateTransparencyLogParams

from cryptodatahub.common.fetcher import FetcherCertificateTransparencyLogs

from updaters.common import UpdaterBase


class UpdaterCertificateTransparencyLogs(UpdaterBase):
    def __init__(self):
        super().__init__(
            FetcherCertificateTransparencyLogs,
            CertificateTransparencyLog,
            CertificateTransparencyLogParams,
        )


def main():
    UpdaterCertificateTransparencyLogs()()  # pragma: no cover


if __name__ == '__main__':
    main()  # pragma: no cover



# -*- coding: utf-8 -*-

import abc
import collections
import enum
import json
import pathlib
import unittest

import attr
import pyfakefs.fake_filesystem_unittest

from cryptodatahub.common.grade import Grade, Gradeable, GradeableSimple, GradeableVulnerabilities
from cryptodatahub.common.key import PublicKey, PublicKeyX509Base
from cryptodatahub.common.types import (
    CryptoDataParamsEnumNumeric,
    CryptoDataParamsEnumString,
    CryptoDataParamsOIDOptional,
    CryptoDataParamsBase,
    CryptoDataParamsNamed,
    convert_dict_to_object,
)


TEST_URL_PREFIX = '/'.join([
    'https://gist.githubusercontent.com',
    'c0r0n3r',
    '54386701406df6e7299bd95c46a4c8d1',
    'raw',
    'e0b9cf606739d3fc1d97cc7f21e501e118bc4e07',
    ''
])


@attr.s
class TestJsonObjectComplex(CryptoDataParamsBase):
    attr = attr.ib()


@attr.s
class TestJsonObject(CryptoDataParamsBase):
    attr_simple = attr.ib()
    attr_iterable = attr.ib()
    attr_complex = attr.ib(converter=convert_dict_to_object(TestJsonObjectComplex))
    no_init_attr = attr.ib(init=False, default=None)
    _attr_hidden = attr.ib(init=False, default=None)


class EnumTest(enum.Enum):
    NONE = 'NONE'
    STRING = 'STRING'
    INT = 'INT'
    FLOAT = 'FLOAT'
    BOOL = 'BOOL'


class EnumIntTest(enum.IntEnum):
    ONE = 1
    TEN = 10


class TestEnumNamedParams(CryptoDataParamsNamed):
    pass


class TestEnumNumericParams(CryptoDataParamsEnumNumeric):
    @classmethod
    def get_code_size(cls):
        return 1


class TestEnumStringParams(CryptoDataParamsEnumString):
    pass


class TestEnumOidParams(CryptoDataParamsOIDOptional):
    pass


class TestGradeableSimple(GradeableSimple):
    @property
    def grade(self):
        return Grade.INSECURE

    def __str__(self):
        return self.grade.value.name


class TestClasses:
    class TestJsonBase(unittest.TestCase):
        @classmethod
        @abc.abstractmethod
        def _get_class(cls):
            raise NotImplementedError()

        @classmethod
        def _get_protected_names(cls):
            return set(['_standard', ])

        @classmethod
        def _get_params_class(cls):
            return type(list(cls._get_class())[0].value)

        def setUp(self):
            self.json_file = open(  # pylint: disable=consider-using-with
                str(self._get_class().get_json_path(self._get_params_class())), 'r', encoding='ascii'
            )
            self.json_data = json.load(self.json_file, object_pairs_hook=collections.OrderedDict)

        def tearDown(self):
            self.json_file.close()

        def test_json_file_format(self):
            wrong_indents = []
            tab_in_lines = []
            wrong_number_of_spces_after_colon = []
            whitespace_only_lines = []
            trailing_whitespaces = []

            self.json_file.seek(0)
            for index, line in enumerate(self.json_file):
                if not line.strip():
                    whitespace_only_lines.append(index + 1)
                try:
                    colon_index = line.index(':')
                    if line[colon_index + 1] != ' ':
                        wrong_number_of_spces_after_colon.append(index + 1)
                    if line[colon_index + 2] == ' ':
                        wrong_number_of_spces_after_colon.append(index + 1)
                except ValueError:
                    pass
                if '\t' in line:
                    tab_in_lines.append(index + 1)
                if line.startswith(' ') and (len(line) - len(line.lstrip())) % 4 != 0:
                    wrong_indents.append(index + 1)
                if len(line) - 1 != len(line.rstrip()):
                    trailing_whitespaces.append(index + 1)

            self.assertEqual(whitespace_only_lines, [])
            self.assertEqual(tab_in_lines, [])
            self.assertEqual(wrong_number_of_spces_after_colon, [])
            self.assertEqual(trailing_whitespaces, [])
            self.assertEqual(wrong_indents, [])

        def test_param_list_len(self):
            self.assertEqual(set(map(lambda item: item.name, self._get_class())), set(self.json_data))

        def test_param_attribute_names_protected(self):
            protected_names = self._get_protected_names()

            self.assertEqual([
                name
                for name, item in self.json_data.items()
                if set(key for key in item.keys() if key.startswith('_')) in protected_names
            ], [])

        def test_param_attribute_order(self):
            attribute_names = self._get_params_class().get_init_attribute_names()

            items_with_surplus_attributes = []
            for name, item in self.json_data.items():
                item_keys = item.keys()
                json_attr_names = [key for key in item_keys if not key.startswith('_')]

                item_keys = set(item_keys)
                param_calss_attr_names = [
                    attribute_name for attribute_name in attribute_names if attribute_name in item_keys
                ]

                if json_attr_names != param_calss_attr_names:
                    items_with_surplus_attributes.append(name)
            self.assertEqual(items_with_surplus_attributes, [])

            attribute_names_not_ordered = []
            for name, item in self.json_data.items():
                protected_count = 0
                for key in item.keys():
                    protected_count += 1
                    if not key.startswith('_'):
                        break

                public_count = 0
                for key in list(item.keys())[protected_count:]:
                    public_count += 1
                    if key.startswith('_'):
                        break

                if protected_count + public_count != len(item.keys()):
                    attribute_names_not_ordered.append(key)

            self.assertEqual(attribute_names_not_ordered, [])

        def test_str_works_for_all_items(self):
            if '__str__' not in dict(self._get_params_class().__dict__.items()):
                return

            for enum_item in self._get_class():
                self.assertTrue(str(enum_item))

        def test_gradeable(self):
            gradeable_class = self._get_params_class()
            if not issubclass(gradeable_class, Gradeable):
                return

            self.assertTrue(
                not issubclass(gradeable_class, GradeableVulnerabilities) or len(gradeable_class.get_gradeable_name())
            )

    class TestJsonCodeStringBase(TestJsonBase):
        @classmethod
        @abc.abstractmethod
        def _get_class(cls):
            raise NotImplementedError()

    class TestJsonCodeNumericBase(TestJsonBase):
        @classmethod
        @abc.abstractmethod
        def _get_class(cls):
            raise NotImplementedError()

        def test_code_size(self):
            self.assertGreater(self._get_params_class().get_code_size(), 0)

        @classmethod
        def _get_protected_names(cls):
            return super(TestClasses.TestJsonCodeNumericBase, cls)._get_protected_names() | set(['_code_in_hex', ])

        def test_param_attribute_value_code_in_hex(self):
            self.assertEqual([
                item['_code_in_hex']
                for item in self.json_data.values()
                if int(item['code']) != int(item['_code_in_hex'], 16)
            ], [])

    class TestKeyBase(pyfakefs.fake_filesystem_unittest.TestCase):
        def setUp(self):
            self.setUpPyfakefs()

            self.__certs_dir = pathlib.PurePath(__file__).parent.parent / 'common' / 'certs'
            self.fs.add_real_directory(str(self.__certs_dir))

        def _get_public_key_pem(self, public_key_file_name):
            public_key_path = self.__certs_dir / (public_key_file_name + '.pem')
            with open(str(public_key_path), 'r', encoding='ascii') as pem_file:
                return pem_file.read()

        def _get_public_key(self, public_key_file_name):
            return PublicKey.from_pem(self._get_public_key_pem(public_key_file_name))

        def _get_public_key_x509(self, public_key_file_name):
            return PublicKeyX509Base.from_pem(self._get_public_key_pem(public_key_file_name))



# -*- coding: utf-8 -*-

import datetime
import dateutil
import urllib3

try:
    from unittest import mock
except ImportError:
    import mock

import pyfakefs.fake_filesystem_unittest

from cryptodatahub.common.types import CryptoDataEnumBase
from cryptodatahub.common.stores import (
    CertificateTransparencyLogParams,
    CertificateTransparencyLogState,
    CertificateTransparencyLogStateType,
    CertificateTransparencyLogTemporalInterval,
)

from cryptodatahub.common.fetcher import FetcherCertificateTransparencyLogs

from updaters.ct_log import UpdaterCertificateTransparencyLogs


class TestUpdaterCertificateTransparencyLogs(pyfakefs.fake_filesystem_unittest.TestCase):
    CT_LOG_LIST_JSON_SAMPLE = """
        {
          "operators": [
            {
              "name": "Up In The Air Consulting",
              "email": [
                "filippo@cloudflare.com"
              ],
              "logs": [
                {
                  "description": "description",
                  "log_id": "bG9nX2lk",
                  "key": "a2V5",
                  "url": "https://log.id/",
                  "mmd": 86400,
                  "state": {
                    "rejected": {
                      "timestamp": "1970-01-01T00:00:00Z"
                    }
                  },
                  "temporal_interval": {
                    "start_inclusive": "1970-01-01T00:00:00Z",
                    "end_exclusive": "1970-01-01T00:00:00Z"
                  }
                }
              ]
            }
          ]
        }
    """
    CT_LOG_ITEM_OBJECT = CertificateTransparencyLogParams(
         log_id='bG9nX2lk',
         operator='FILIPPO_VALSORDA',
         key='a2V5',
         url='https://log.id/',
         mmd=86400,
         description='description',
         temporal_interval=CertificateTransparencyLogTemporalInterval(
            start_inclusive=datetime.datetime(1970, 1, 1, 0, 0, 0, tzinfo=dateutil.tz.UTC),
            end_exclusive=datetime.datetime(1970, 1, 1, 0, 0, 0, tzinfo=dateutil.tz.UTC),
         ),
         log_state=CertificateTransparencyLogState(
            state_type=CertificateTransparencyLogStateType.REJECTED,
            timestamp=datetime.datetime(1970, 1, 1, 0, 0, 0, tzinfo=dateutil.tz.UTC),
         ),
    )

    def setUp(self):
        self.setUpPyfakefs()

    @mock.patch.object(urllib3.poolmanager.PoolManager, 'request')
    def test_log_list_parsing(self, mock_request):
        mock_request.return_value = urllib3.response.HTTPResponse(self.CT_LOG_LIST_JSON_SAMPLE)
        current_data = FetcherCertificateTransparencyLogs.from_current_data()
        self.assertTrue(isinstance(current_data, FetcherCertificateTransparencyLogs))
        self.assertEqual(current_data.parsed_data[0], self.CT_LOG_ITEM_OBJECT)

    @mock.patch.object(urllib3.poolmanager.PoolManager, 'request')
    def test_log_list_update(self, mock_request):
        mock_request.return_value = urllib3.response.HTTPResponse(self.CT_LOG_LIST_JSON_SAMPLE)
        json_file_path = str(CryptoDataEnumBase.get_json_path(CertificateTransparencyLogParams))
        self.fs.create_file(json_file_path, contents='{}')

        UpdaterCertificateTransparencyLogs()()
        with open(json_file_path, 'r', encoding='ascii') as json_file:
            self.assertEqual(
                json_file.read(),
                CryptoDataEnumBase.dump_json({'DESCRIPTION': self.CT_LOG_ITEM_OBJECT._asdict()}),
            )



# -*- coding: utf-8 -*-

import unittest

from test.common.classes import TestClasses

from cryptodatahub.common.algorithm import (
    Authentication,
    BlockCipher,
    BlockCipherMode,
    Hash,
    KeyExchange,
    NamedGroup,
    MAC,
    MACModeParams,
    MACParams,
    Signature,
)


class TestAuthentication(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return Authentication


class TestBlockCipher(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return BlockCipher


class TestBlockCipherMode(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return BlockCipherMode


class TestHash(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return Hash


class TestKeyExchange(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return KeyExchange


class TestNamedGroup(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return NamedGroup


class TestMAC(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return MAC


class TestMACParams(unittest.TestCase):
    def test_error_both_attributes_are_null(self):
        with self.assertRaises(ValueError):
            MACParams('name', 'long_name', None, None, None, None)


class TestMACModeParams(unittest.TestCase):
    def test_gradeable(self):
        self.assertTrue(len(MACModeParams.get_gradeable_name()))


class TestSignature(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return Signature



# -*- coding: utf-8 -*-

import unittest

from test.common.classes import TEST_URL_PREFIX

import attr

from cryptodatahub.common.fetcher import FetcherBase, FetcherCsvBase
from cryptodatahub.common.utils import HttpFetcher, name_to_enum_item_name


@attr.s
class FetcherBaseTest(FetcherBase):
    @classmethod
    def _get_current_data(cls):
        return b'data'

    @classmethod
    def _transform_data(cls, current_data):
        return {name_to_enum_item_name('data'): current_data.decode('ascii')}


class TestFetcherBase(unittest.TestCase):
    def test_current_data(self):
        self.assertEqual(FetcherBaseTest.from_current_data(), FetcherBaseTest({'DATA': 'data'}))


@attr.s
class FetcherCsvBaseTest(FetcherCsvBase):
    @classmethod
    def _get_csv_url(cls):
        return TEST_URL_PREFIX + 'test.csv'

    @classmethod
    def _get_csv_fields(cls):
        return ['Col 1', 'Col 2']

    @classmethod
    def _get_fetcher(cls):
        return HttpFetcher()

    @classmethod
    def _transform_data(cls, current_data):
        return list(map(dict, current_data))


class TestFetcherCsvBase(unittest.TestCase):
    def test_current_data(self):
        self.assertEqual(
            FetcherCsvBaseTest.from_current_data(),
            FetcherCsvBaseTest([
                {'Col 1': 'Row 1, Col 1', 'Col 2': 'Row 1, Col 2'},
                {'Col 1': 'Row 2, Col 1', 'Col 2': 'Row 2, Col 2'},
            ])
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import abc
import collections
import csv
import datetime
import io
import json
import tarfile

import attr
import bs4
import urllib3

from cryptodatahub.common.entity import Entity
from cryptodatahub.common.stores import (
    CertificateTransparencyLogParams,
    CertificateTrustConstraint,
    RootCertificateParams,
    RootCertificateTrustConstraintAction,
    RootCertificateTrustStoreConstraint,
)
from cryptodatahub.common.stores import RootCertificate
from cryptodatahub.common.utils import HttpFetcher, name_to_enum_item_name


@attr.s(frozen=True)
class CertificatePemFetcher():
    http_fetcher = attr.ib(
        init=False,
        default=HttpFetcher(connect_timeout=5, read_timeout=30, retry=10),
        validator=attr.validators.instance_of(HttpFetcher),
    )

    def __call__(self, sha2_256_fingerprint):
        try:
            return RootCertificate.get_item_by_sha2_256_fingerprint(
                sha2_256_fingerprint
            ).value.certificate.pem
        except KeyError:
            data = self.http_fetcher(f'https://crt.sh/?d={sha2_256_fingerprint}')
            return data.decode('utf-8').strip()


@attr.s
class FetcherBase():
    """Abstract base class for all data fetchers.

    Subclasses must implement:
    - _get_current_data(): Fetch raw data from source
    - _transform_data(): Transform raw data into parsed format
    """
    parsed_data = attr.ib(validator=attr.validators.instance_of(collections.abc.Iterable))

    @classmethod
    @abc.abstractmethod
    def _get_current_data(cls):
        """Fetch raw data from the data source.

        Returns:
            Raw data in source-specific format
        """
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def _transform_data(cls, current_data):
        """Transform raw data into parsed format.

        Args:
            current_data: Raw data from _get_current_data()

        Returns:
            Transformed/parsed data
        """
        raise NotImplementedError()

    @classmethod
    def from_current_data(cls):
        """Fetch and parse data from the source.

        Returns:
            Instance of the fetcher class with parsed_data populated
        """
        current_data = cls._get_current_data()
        transformed_data = cls._transform_data(current_data)

        return cls(transformed_data)


@attr.s
class FetcherCertificateTransparencyLogs(FetcherBase):
    _CT_LOGS_ALL_JSON_URL = 'https://www.gstatic.com/ct/log_list/v3/all_logs_list.json'
    _CT_LOG_OPERATOR_NAME_GOOGLE_NAME = {
        'Up In The Air Consulting': 'Filippo Valsorda',
        'Wang Shengnan': 'GDCA',
    }

    @classmethod
    def _get_current_data(cls):
        http = urllib3.PoolManager()
        response = http.request('GET', cls._CT_LOGS_ALL_JSON_URL, preload_content=False)
        response.release_conn()

        return json.loads(response.data, object_pairs_hook=collections.OrderedDict)

    @classmethod
    def _transform_data(cls, current_data):
        transformed_logs = []

        for operator in current_data['operators']:
            logs = operator.pop('logs')
            for log in logs:
                if 'state' in log:
                    state = log.pop('state')
                    state_type = list(state)[0]
                    state_args = collections.OrderedDict([('state_type', state_type)])
                    state_args.update(state[state_type])
                    state_args.pop('final_tree_head', None)

                    log.update(collections.OrderedDict([('log_state', state_args)]))

                operator_name = operator['name']
                operator_name = cls._CT_LOG_OPERATOR_NAME_GOOGLE_NAME.get(operator_name, operator_name)
                log['operator'] = Entity[name_to_enum_item_name(operator_name)].name

                transformed_logs.append(CertificateTransparencyLogParams(**log))

        return transformed_logs


@attr.s
class FetcherCsvBase(FetcherBase):
    """Base class for CSV-based data fetchers.

    Subclasses must implement:
    - _get_csv_url(): Return the URL of the CSV file
    - _get_csv_fields(): Return the field names for the CSV
    - _get_fetcher(): Return an HTTP fetcher instance
    - _transform_data(): Transform CSV rows into parsed format
    """

    @classmethod
    @abc.abstractmethod
    def _get_csv_url(cls):
        """Get the URL of the CSV file to fetch.

        Returns:
            str: URL of the CSV file
        """
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def _get_csv_fields(cls):
        """Get the field names for the CSV file.

        Returns:
            tuple: Field names
        """
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def _get_fetcher(cls):
        """Get an HTTP fetcher instance.

        Returns:
            An HTTP fetcher object with a __call__ method that takes a URL
        """
        raise NotImplementedError()

    @classmethod
    def _get_current_data(cls):
        """Fetch and parse CSV data.

        Returns:
            csv.DictReader: CSV reader with parsed rows
        """
        data = cls._get_fetcher()(cls._get_csv_url())

        csv_reader = csv.DictReader(
            io.StringIO(data.decode('utf-8')),
            fieldnames=cls._get_csv_fields(),
        )

        sample = data[:4096].decode('utf-8')
        if csv.Sniffer().has_header(sample):
            next(csv_reader, None)

        return csv_reader

    @classmethod
    @abc.abstractmethod
    def _transform_data(cls, current_data):
        """Transform CSV rows into parsed format.

        Args:
            current_data: CSV reader from _get_current_data()

        Returns:
            Transformed/parsed data
        """
        raise NotImplementedError()


class FetcherRootCertificateStoreGoogle(FetcherBase):
    @classmethod
    def _get_current_data(cls):
        data = HttpFetcher()(
            'https://android.googlesource.com/platform/system/ca-certificates/+archive/refs/heads/master/files.tar.gz'
        )
        with tarfile.open(fileobj=io.BytesIO(data), mode='r') as tar:
            for member in tar.getmembers():
                yield tar.extractfile(member).read().decode('ascii')

    @classmethod
    def _transform_data(cls, current_data):
        certificates = {}
        for root_certificate_pem in current_data:
            root_certificate_pem_lines = root_certificate_pem.splitlines()
            root_certificate_pem_lines = root_certificate_pem_lines[
                :root_certificate_pem_lines.index('-----END CERTIFICATE-----') + 1
            ]

            certificates[tuple(root_certificate_pem_lines)] = tuple()

        return certificates


class FetcherRootCertificateStoreApple(FetcherBase):
    FIELDS = (
        'Certificate name',
        'Issued by',
        'Type',
        'Key size',
        'Sig alg',
        'Serial number',
        'Expires',
        'EV policy',
        'Fingerprint (SHA-256)',
    )

    @classmethod
    def _get_current_data(cls):
        response = HttpFetcher()('https://support.apple.com/en-us/HT213080')
        page = bs4.BeautifulSoup(response.decode('utf-8'), 'html.parser')
        trusted_cas = page.find("h2", {"id": "trusted"})
        return [
            dict(zip(cls.FIELDS, map(lambda value: value.text.strip(), row.find_all('td'))))
            for row in trusted_cas.find_next('tbody').find_all('tr')[1:]
        ]

    @classmethod
    def _transform_data(cls, current_data):
        certificate_pem_fetcher = CertificatePemFetcher()
        certificates = {}
        for row in current_data:
            sha2_256_fingerprint = row['Fingerprint (SHA-256)'].replace(' ', '')
            root_certificate_pem = certificate_pem_fetcher(sha2_256_fingerprint)
            certificates[tuple(root_certificate_pem.splitlines())] = ()

        return certificates


class FetcherRootCertificateStoreMicrosoft(FetcherCsvBase):
    CSV_FIELDS = (
        'Microsoft Status',
        'CA Owner',
        'CA Common Name or Certificate Name',
        'SHA-1 Fingerprint',
        'SHA-256 Fingerprint',
        'Microsoft EKUs',
        'Valid From [GMT]',
        'Valid To [GMT]',
        'Public Key Algorithm',
        'Signature Hash Algorithm',
    )

    @classmethod
    def _get_csv_url(cls):
        return 'https://ccadb-public.secure.force.com/microsoft/IncludedCACertificateReportForMSFTCSV'

    @classmethod
    def _get_csv_fields(cls):
        return cls.CSV_FIELDS

    @classmethod
    def _get_fetcher(cls):
        return HttpFetcher()

    @classmethod
    def _transform_data(cls, current_data):
        certificate_pem_fetcher = CertificatePemFetcher()
        certificates = {}

        for row in current_data:
            status = row['Microsoft Status']

            if status == 'Disabled':
                continue

            constraints = []
            if status == 'NotBefore':
                constraint = CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    date=datetime.datetime.strptime(row['Valid From [GMT]'], '%Y %b %d'),
                )
                constraints.append(constraint)

            sha2_256_fingerprint = row['SHA-256 Fingerprint']
            root_certificate_pem = certificate_pem_fetcher(sha2_256_fingerprint)

            certificates[tuple(root_certificate_pem.splitlines())] = tuple(constraints)

        return certificates


class FetcherRootCertificateStoreMozilla(FetcherCsvBase):
    CSV_FIELDS = ('PEM', 'Distrust for TLS After Date', 'Mozilla Applied Constraints')

    @classmethod
    def _get_csv_url(cls):
        return \
            'https://ccadb.my.salesforce-sites.com/mozilla/IncludedRootsDistrustTLSSSLPEMCSV?TrustBitsInclude=Websites'

    @classmethod
    def _get_csv_fields(cls):
        return cls.CSV_FIELDS

    @classmethod
    def _get_fetcher(cls):
        return HttpFetcher()

    @classmethod
    def _transform_data(cls, current_data):
        certificates = {}

        for row in current_data:
            constraints = []

            distrust_after = row['Distrust for TLS After Date']
            if distrust_after:
                constraint = CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    date=datetime.datetime.strptime(distrust_after, '%Y.%m.%d'),
                )
                constraints.append(constraint)

            domain = row['Mozilla Applied Constraints']
            if domain:
                constraint = CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    domains=[domain],
                )
                constraints.append(constraint)

            root_certificate_pem_lines = tuple(row['PEM'].strip('\'').splitlines())
            certificates[root_certificate_pem_lines] = tuple(constraints)

        return certificates


class FetcherRootCertificateStore(FetcherBase):
    _ROOT_CERTIFICATE_STORE_UPDATERS = collections.OrderedDict([
        (Entity.MOZILLA, FetcherRootCertificateStoreMozilla),
        (Entity.GOOGLE, FetcherRootCertificateStoreGoogle),
        (Entity.MICROSOFT, FetcherRootCertificateStoreMicrosoft),
        (Entity.APPLE, FetcherRootCertificateStoreApple),
    ])

    @classmethod
    def _get_current_data(cls):
        return collections.OrderedDict([
            (store_owner, store_fetcher_class.from_current_data())
            for store_owner, store_fetcher_class in cls._ROOT_CERTIFICATE_STORE_UPDATERS.items()
        ])

    @classmethod
    def _transform_data(cls, current_data):
        root_certificates = {}

        for store_owner, root_store in current_data.items():
            for certificate_data, constraints in root_store.parsed_data.items():
                if certificate_data in root_certificates:
                    merged_constraints = root_certificates[certificate_data]
                else:
                    merged_constraints = []
                    root_certificates[certificate_data] = merged_constraints

                merged_constraints.append(RootCertificateTrustStoreConstraint(store_owner, constraints))

        return [
            RootCertificateParams(*item)
            for item in root_certificates.items()
        ]
